# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.25.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class DeviceEnum(StrEnum):
    DESKTOP = "DESKTOP"
    MOBILE = "MOBILE"


class SizeEnum(StrEnum):
    LARGE = "LARGE"
    MEDIUM = "MEDIUM"
    SMALL = "SMALL"


class ThemeEnum(StrEnum):
    DARK = "DARK"
    DEFAULT = "DEFAULT"
    DEFAULT_DARK = "DEFAULT_DARK"


class XsollaPaywallConfigRequest(Model):
    """Xsolla paywall config request (XsollaPaywallConfigRequest)

    Properties:
        device: (device) OPTIONAL Union[str, DeviceEnum]

        show_close_button: (showCloseButton) OPTIONAL bool

        size: (size) OPTIONAL Union[str, SizeEnum]

        theme: (theme) OPTIONAL Union[str, ThemeEnum]
    """

    # region fields

    device: Union[str, DeviceEnum]  # OPTIONAL
    show_close_button: bool  # OPTIONAL
    size: Union[str, SizeEnum]  # OPTIONAL
    theme: Union[str, ThemeEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_device(self, value: Union[str, DeviceEnum]) -> XsollaPaywallConfigRequest:
        self.device = value
        return self

    def with_show_close_button(self, value: bool) -> XsollaPaywallConfigRequest:
        self.show_close_button = value
        return self

    def with_size(self, value: Union[str, SizeEnum]) -> XsollaPaywallConfigRequest:
        self.size = value
        return self

    def with_theme(self, value: Union[str, ThemeEnum]) -> XsollaPaywallConfigRequest:
        self.theme = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "device"):
            result["device"] = str(self.device)
        elif include_empty:
            result["device"] = Union[str, DeviceEnum]()
        if hasattr(self, "show_close_button"):
            result["showCloseButton"] = bool(self.show_close_button)
        elif include_empty:
            result["showCloseButton"] = False
        if hasattr(self, "size"):
            result["size"] = str(self.size)
        elif include_empty:
            result["size"] = Union[str, SizeEnum]()
        if hasattr(self, "theme"):
            result["theme"] = str(self.theme)
        elif include_empty:
            result["theme"] = Union[str, ThemeEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        device: Optional[Union[str, DeviceEnum]] = None,
        show_close_button: Optional[bool] = None,
        size: Optional[Union[str, SizeEnum]] = None,
        theme: Optional[Union[str, ThemeEnum]] = None,
    ) -> XsollaPaywallConfigRequest:
        instance = cls()
        if device is not None:
            instance.device = device
        if show_close_button is not None:
            instance.show_close_button = show_close_button
        if size is not None:
            instance.size = size
        if theme is not None:
            instance.theme = theme
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> XsollaPaywallConfigRequest:
        instance = cls()
        if not dict_:
            return instance
        if "device" in dict_ and dict_["device"] is not None:
            instance.device = str(dict_["device"])
        elif include_empty:
            instance.device = Union[str, DeviceEnum]()
        if "showCloseButton" in dict_ and dict_["showCloseButton"] is not None:
            instance.show_close_button = bool(dict_["showCloseButton"])
        elif include_empty:
            instance.show_close_button = False
        if "size" in dict_ and dict_["size"] is not None:
            instance.size = str(dict_["size"])
        elif include_empty:
            instance.size = Union[str, SizeEnum]()
        if "theme" in dict_ and dict_["theme"] is not None:
            instance.theme = str(dict_["theme"])
        elif include_empty:
            instance.theme = Union[str, ThemeEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, XsollaPaywallConfigRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[XsollaPaywallConfigRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        XsollaPaywallConfigRequest,
        List[XsollaPaywallConfigRequest],
        Dict[Any, XsollaPaywallConfigRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "device": "device",
            "showCloseButton": "show_close_button",
            "size": "size",
            "theme": "theme",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "device": False,
            "showCloseButton": False,
            "size": False,
            "theme": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "device": ["DESKTOP", "MOBILE"],
            "size": ["LARGE", "MEDIUM", "SMALL"],
            "theme": ["DARK", "DEFAULT", "DEFAULT_DARK"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.25.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class CycleEnum(StrEnum):
    MONTHLY = "MONTHLY"
    QUARTERLY = "QUARTERLY"
    WEEKLY = "WEEKLY"
    YEARLY = "YEARLY"


class Recurring(Model):
    """Recurring (Recurring)

    Properties:
        cycle: (cycle) REQUIRED Union[str, CycleEnum]

        fixed_free_days: (fixedFreeDays) REQUIRED int

        fixed_trial_cycles: (fixedTrialCycles) REQUIRED int

        grace_days: (graceDays) REQUIRED int
    """

    # region fields

    cycle: Union[str, CycleEnum]  # REQUIRED
    fixed_free_days: int  # REQUIRED
    fixed_trial_cycles: int  # REQUIRED
    grace_days: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_cycle(self, value: Union[str, CycleEnum]) -> Recurring:
        self.cycle = value
        return self

    def with_fixed_free_days(self, value: int) -> Recurring:
        self.fixed_free_days = value
        return self

    def with_fixed_trial_cycles(self, value: int) -> Recurring:
        self.fixed_trial_cycles = value
        return self

    def with_grace_days(self, value: int) -> Recurring:
        self.grace_days = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "cycle"):
            result["cycle"] = str(self.cycle)
        elif include_empty:
            result["cycle"] = Union[str, CycleEnum]()
        if hasattr(self, "fixed_free_days"):
            result["fixedFreeDays"] = int(self.fixed_free_days)
        elif include_empty:
            result["fixedFreeDays"] = 0
        if hasattr(self, "fixed_trial_cycles"):
            result["fixedTrialCycles"] = int(self.fixed_trial_cycles)
        elif include_empty:
            result["fixedTrialCycles"] = 0
        if hasattr(self, "grace_days"):
            result["graceDays"] = int(self.grace_days)
        elif include_empty:
            result["graceDays"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        cycle: Union[str, CycleEnum],
        fixed_free_days: int,
        fixed_trial_cycles: int,
        grace_days: int,
    ) -> Recurring:
        instance = cls()
        instance.cycle = cycle
        instance.fixed_free_days = fixed_free_days
        instance.fixed_trial_cycles = fixed_trial_cycles
        instance.grace_days = grace_days
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> Recurring:
        instance = cls()
        if not dict_:
            return instance
        if "cycle" in dict_ and dict_["cycle"] is not None:
            instance.cycle = str(dict_["cycle"])
        elif include_empty:
            instance.cycle = Union[str, CycleEnum]()
        if "fixedFreeDays" in dict_ and dict_["fixedFreeDays"] is not None:
            instance.fixed_free_days = int(dict_["fixedFreeDays"])
        elif include_empty:
            instance.fixed_free_days = 0
        if "fixedTrialCycles" in dict_ and dict_["fixedTrialCycles"] is not None:
            instance.fixed_trial_cycles = int(dict_["fixedTrialCycles"])
        elif include_empty:
            instance.fixed_trial_cycles = 0
        if "graceDays" in dict_ and dict_["graceDays"] is not None:
            instance.grace_days = int(dict_["graceDays"])
        elif include_empty:
            instance.grace_days = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, Recurring]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[Recurring]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[Recurring, List[Recurring], Dict[Any, Recurring]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "cycle": "cycle",
            "fixedFreeDays": "fixed_free_days",
            "fixedTrialCycles": "fixed_trial_cycles",
            "graceDays": "grace_days",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "cycle": True,
            "fixedFreeDays": True,
            "fixedTrialCycles": True,
            "graceDays": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "cycle": ["MONTHLY", "QUARTERLY", "WEEKLY", "YEARLY"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (5.29.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_device_ban_response_v4 import ModelDeviceBanResponseV4


class ModelDeviceResponseV4(Model):
    """Model device response V4 (model.DeviceResponseV4)

    Properties:
        device_id: (deviceId) REQUIRED str

        device_type: (deviceType) REQUIRED str

        ip: (ip) REQUIRED str

        last_login_time: (lastLoginTime) REQUIRED int

        ban: (ban) OPTIONAL ModelDeviceBanResponseV4

        ext: (ext) OPTIONAL Dict[str, Any]
    """

    # region fields

    device_id: str  # REQUIRED
    device_type: str  # REQUIRED
    ip: str  # REQUIRED
    last_login_time: int  # REQUIRED
    ban: ModelDeviceBanResponseV4  # OPTIONAL
    ext: Dict[str, Any]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_device_id(self, value: str) -> ModelDeviceResponseV4:
        self.device_id = value
        return self

    def with_device_type(self, value: str) -> ModelDeviceResponseV4:
        self.device_type = value
        return self

    def with_ip(self, value: str) -> ModelDeviceResponseV4:
        self.ip = value
        return self

    def with_last_login_time(self, value: int) -> ModelDeviceResponseV4:
        self.last_login_time = value
        return self

    def with_ban(self, value: ModelDeviceBanResponseV4) -> ModelDeviceResponseV4:
        self.ban = value
        return self

    def with_ext(self, value: Dict[str, Any]) -> ModelDeviceResponseV4:
        self.ext = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "device_id"):
            result["deviceId"] = str(self.device_id)
        elif include_empty:
            result["deviceId"] = ""
        if hasattr(self, "device_type"):
            result["deviceType"] = str(self.device_type)
        elif include_empty:
            result["deviceType"] = ""
        if hasattr(self, "ip"):
            result["ip"] = str(self.ip)
        elif include_empty:
            result["ip"] = ""
        if hasattr(self, "last_login_time"):
            result["lastLoginTime"] = int(self.last_login_time)
        elif include_empty:
            result["lastLoginTime"] = 0
        if hasattr(self, "ban"):
            result["ban"] = self.ban.to_dict(include_empty=include_empty)
        elif include_empty:
            result["ban"] = ModelDeviceBanResponseV4()
        if hasattr(self, "ext"):
            result["ext"] = {str(k0): v0 for k0, v0 in self.ext.items()}
        elif include_empty:
            result["ext"] = {}
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        device_id: str,
        device_type: str,
        ip: str,
        last_login_time: int,
        ban: Optional[ModelDeviceBanResponseV4] = None,
        ext: Optional[Dict[str, Any]] = None,
    ) -> ModelDeviceResponseV4:
        instance = cls()
        instance.device_id = device_id
        instance.device_type = device_type
        instance.ip = ip
        instance.last_login_time = last_login_time
        if ban is not None:
            instance.ban = ban
        if ext is not None:
            instance.ext = ext
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelDeviceResponseV4:
        instance = cls()
        if not dict_:
            return instance
        if "deviceId" in dict_ and dict_["deviceId"] is not None:
            instance.device_id = str(dict_["deviceId"])
        elif include_empty:
            instance.device_id = ""
        if "deviceType" in dict_ and dict_["deviceType"] is not None:
            instance.device_type = str(dict_["deviceType"])
        elif include_empty:
            instance.device_type = ""
        if "ip" in dict_ and dict_["ip"] is not None:
            instance.ip = str(dict_["ip"])
        elif include_empty:
            instance.ip = ""
        if "lastLoginTime" in dict_ and dict_["lastLoginTime"] is not None:
            instance.last_login_time = int(dict_["lastLoginTime"])
        elif include_empty:
            instance.last_login_time = 0
        if "ban" in dict_ and dict_["ban"] is not None:
            instance.ban = ModelDeviceBanResponseV4.create_from_dict(
                dict_["ban"], include_empty=include_empty
            )
        elif include_empty:
            instance.ban = ModelDeviceBanResponseV4()
        if "ext" in dict_ and dict_["ext"] is not None:
            instance.ext = {str(k0): v0 for k0, v0 in dict_["ext"].items()}
        elif include_empty:
            instance.ext = {}
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelDeviceResponseV4]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelDeviceResponseV4]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelDeviceResponseV4,
        List[ModelDeviceResponseV4],
        Dict[Any, ModelDeviceResponseV4],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "deviceId": "device_id",
            "deviceType": "device_type",
            "ip": "ip",
            "lastLoginTime": "last_login_time",
            "ban": "ban",
            "ext": "ext",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "deviceId": True,
            "deviceType": True,
            "ip": True,
            "lastLoginTime": True,
            "ban": False,
            "ext": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.25.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.currency_summary import CurrencySummary


class InvoiceCurrencySummary(Model):
    """Invoice currency summary (InvoiceCurrencySummary)

    Properties:
        currency: (currency) REQUIRED CurrencySummary

        sales_volume: (salesVolume) REQUIRED int

        subtotal_price: (subtotalPrice) REQUIRED int

        total_price: (totalPrice) REQUIRED int

        total_tax: (totalTax) REQUIRED int
    """

    # region fields

    currency: CurrencySummary  # REQUIRED
    sales_volume: int  # REQUIRED
    subtotal_price: int  # REQUIRED
    total_price: int  # REQUIRED
    total_tax: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_currency(self, value: CurrencySummary) -> InvoiceCurrencySummary:
        self.currency = value
        return self

    def with_sales_volume(self, value: int) -> InvoiceCurrencySummary:
        self.sales_volume = value
        return self

    def with_subtotal_price(self, value: int) -> InvoiceCurrencySummary:
        self.subtotal_price = value
        return self

    def with_total_price(self, value: int) -> InvoiceCurrencySummary:
        self.total_price = value
        return self

    def with_total_tax(self, value: int) -> InvoiceCurrencySummary:
        self.total_tax = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "currency"):
            result["currency"] = self.currency.to_dict(include_empty=include_empty)
        elif include_empty:
            result["currency"] = CurrencySummary()
        if hasattr(self, "sales_volume"):
            result["salesVolume"] = int(self.sales_volume)
        elif include_empty:
            result["salesVolume"] = 0
        if hasattr(self, "subtotal_price"):
            result["subtotalPrice"] = int(self.subtotal_price)
        elif include_empty:
            result["subtotalPrice"] = 0
        if hasattr(self, "total_price"):
            result["totalPrice"] = int(self.total_price)
        elif include_empty:
            result["totalPrice"] = 0
        if hasattr(self, "total_tax"):
            result["totalTax"] = int(self.total_tax)
        elif include_empty:
            result["totalTax"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        currency: CurrencySummary,
        sales_volume: int,
        subtotal_price: int,
        total_price: int,
        total_tax: int,
    ) -> InvoiceCurrencySummary:
        instance = cls()
        instance.currency = currency
        instance.sales_volume = sales_volume
        instance.subtotal_price = subtotal_price
        instance.total_price = total_price
        instance.total_tax = total_tax
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> InvoiceCurrencySummary:
        instance = cls()
        if not dict_:
            return instance
        if "currency" in dict_ and dict_["currency"] is not None:
            instance.currency = CurrencySummary.create_from_dict(
                dict_["currency"], include_empty=include_empty
            )
        elif include_empty:
            instance.currency = CurrencySummary()
        if "salesVolume" in dict_ and dict_["salesVolume"] is not None:
            instance.sales_volume = int(dict_["salesVolume"])
        elif include_empty:
            instance.sales_volume = 0
        if "subtotalPrice" in dict_ and dict_["subtotalPrice"] is not None:
            instance.subtotal_price = int(dict_["subtotalPrice"])
        elif include_empty:
            instance.subtotal_price = 0
        if "totalPrice" in dict_ and dict_["totalPrice"] is not None:
            instance.total_price = int(dict_["totalPrice"])
        elif include_empty:
            instance.total_price = 0
        if "totalTax" in dict_ and dict_["totalTax"] is not None:
            instance.total_tax = int(dict_["totalTax"])
        elif include_empty:
            instance.total_tax = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, InvoiceCurrencySummary]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[InvoiceCurrencySummary]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        InvoiceCurrencySummary,
        List[InvoiceCurrencySummary],
        Dict[Any, InvoiceCurrencySummary],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currency": "currency",
            "salesVolume": "sales_volume",
            "subtotalPrice": "subtotal_price",
            "totalPrice": "total_price",
            "totalTax": "total_tax",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currency": True,
            "salesVolume": True,
            "subtotalPrice": True,
            "totalPrice": True,
            "totalTax": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (5.29.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_namespace_role import AccountcommonNamespaceRole


class ModelUserInvitationV3(Model):
    """Model user invitation V3 (model.UserInvitationV3)

    Properties:
        email: (email) REQUIRED str

        expired_at: (expiredAt) REQUIRED str

        roles: (roles) REQUIRED List[AccountcommonNamespaceRole]

        id_: (id) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        namespace_display_name: (namespaceDisplayName) OPTIONAL str
    """

    # region fields

    email: str  # REQUIRED
    expired_at: str  # REQUIRED
    roles: List[AccountcommonNamespaceRole]  # REQUIRED
    id_: str  # OPTIONAL
    namespace: str  # OPTIONAL
    namespace_display_name: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_email(self, value: str) -> ModelUserInvitationV3:
        self.email = value
        return self

    def with_expired_at(self, value: str) -> ModelUserInvitationV3:
        self.expired_at = value
        return self

    def with_roles(
        self, value: List[AccountcommonNamespaceRole]
    ) -> ModelUserInvitationV3:
        self.roles = value
        return self

    def with_id(self, value: str) -> ModelUserInvitationV3:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> ModelUserInvitationV3:
        self.namespace = value
        return self

    def with_namespace_display_name(self, value: str) -> ModelUserInvitationV3:
        self.namespace_display_name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "email"):
            result["email"] = str(self.email)
        elif include_empty:
            result["email"] = ""
        if hasattr(self, "expired_at"):
            result["expiredAt"] = str(self.expired_at)
        elif include_empty:
            result["expiredAt"] = ""
        if hasattr(self, "roles"):
            result["roles"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.roles
            ]
        elif include_empty:
            result["roles"] = []
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "namespace_display_name"):
            result["namespaceDisplayName"] = str(self.namespace_display_name)
        elif include_empty:
            result["namespaceDisplayName"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        email: str,
        expired_at: str,
        roles: List[AccountcommonNamespaceRole],
        id_: Optional[str] = None,
        namespace: Optional[str] = None,
        namespace_display_name: Optional[str] = None,
    ) -> ModelUserInvitationV3:
        instance = cls()
        instance.email = email
        instance.expired_at = expired_at
        instance.roles = roles
        if id_ is not None:
            instance.id_ = id_
        if namespace is not None:
            instance.namespace = namespace
        if namespace_display_name is not None:
            instance.namespace_display_name = namespace_display_name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelUserInvitationV3:
        instance = cls()
        if not dict_:
            return instance
        if "email" in dict_ and dict_["email"] is not None:
            instance.email = str(dict_["email"])
        elif include_empty:
            instance.email = ""
        if "expiredAt" in dict_ and dict_["expiredAt"] is not None:
            instance.expired_at = str(dict_["expiredAt"])
        elif include_empty:
            instance.expired_at = ""
        if "roles" in dict_ and dict_["roles"] is not None:
            instance.roles = [
                AccountcommonNamespaceRole.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["roles"]
            ]
        elif include_empty:
            instance.roles = []
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if (
            "namespaceDisplayName" in dict_
            and dict_["namespaceDisplayName"] is not None
        ):
            instance.namespace_display_name = str(dict_["namespaceDisplayName"])
        elif include_empty:
            instance.namespace_display_name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelUserInvitationV3]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelUserInvitationV3]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelUserInvitationV3,
        List[ModelUserInvitationV3],
        Dict[Any, ModelUserInvitationV3],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "email": "email",
            "expiredAt": "expired_at",
            "roles": "roles",
            "id": "id_",
            "namespace": "namespace",
            "namespaceDisplayName": "namespace_display_name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "email": True,
            "expiredAt": True,
            "roles": True,
            "id": False,
            "namespace": False,
            "namespaceDisplayName": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service (5.29.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelUserBaseInfo(Model):
    """Model user base info (model.UserBaseInfo)

    Properties:
        avatar_url: (avatarUrl) REQUIRED str

        display_name: (displayName) REQUIRED str

        platform_user_ids: (platformUserIds) REQUIRED Dict[str, str]

        user_id: (userId) REQUIRED str

        username: (username) OPTIONAL str
    """

    # region fields

    avatar_url: str  # REQUIRED
    display_name: str  # REQUIRED
    platform_user_ids: Dict[str, str]  # REQUIRED
    user_id: str  # REQUIRED
    username: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_avatar_url(self, value: str) -> ModelUserBaseInfo:
        self.avatar_url = value
        return self

    def with_display_name(self, value: str) -> ModelUserBaseInfo:
        self.display_name = value
        return self

    def with_platform_user_ids(self, value: Dict[str, str]) -> ModelUserBaseInfo:
        self.platform_user_ids = value
        return self

    def with_user_id(self, value: str) -> ModelUserBaseInfo:
        self.user_id = value
        return self

    def with_username(self, value: str) -> ModelUserBaseInfo:
        self.username = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "avatar_url"):
            result["avatarUrl"] = str(self.avatar_url)
        elif include_empty:
            result["avatarUrl"] = ""
        if hasattr(self, "display_name"):
            result["displayName"] = str(self.display_name)
        elif include_empty:
            result["displayName"] = ""
        if hasattr(self, "platform_user_ids"):
            result["platformUserIds"] = {
                str(k0): str(v0) for k0, v0 in self.platform_user_ids.items()
            }
        elif include_empty:
            result["platformUserIds"] = {}
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "username"):
            result["username"] = str(self.username)
        elif include_empty:
            result["username"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        avatar_url: str,
        display_name: str,
        platform_user_ids: Dict[str, str],
        user_id: str,
        username: Optional[str] = None,
    ) -> ModelUserBaseInfo:
        instance = cls()
        instance.avatar_url = avatar_url
        instance.display_name = display_name
        instance.platform_user_ids = platform_user_ids
        instance.user_id = user_id
        if username is not None:
            instance.username = username
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelUserBaseInfo:
        instance = cls()
        if not dict_:
            return instance
        if "avatarUrl" in dict_ and dict_["avatarUrl"] is not None:
            instance.avatar_url = str(dict_["avatarUrl"])
        elif include_empty:
            instance.avatar_url = ""
        if "displayName" in dict_ and dict_["displayName"] is not None:
            instance.display_name = str(dict_["displayName"])
        elif include_empty:
            instance.display_name = ""
        if "platformUserIds" in dict_ and dict_["platformUserIds"] is not None:
            instance.platform_user_ids = {
                str(k0): str(v0) for k0, v0 in dict_["platformUserIds"].items()
            }
        elif include_empty:
            instance.platform_user_ids = {}
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "username" in dict_ and dict_["username"] is not None:
            instance.username = str(dict_["username"])
        elif include_empty:
            instance.username = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelUserBaseInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelUserBaseInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelUserBaseInfo, List[ModelUserBaseInfo], Dict[Any, ModelUserBaseInfo]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "avatarUrl": "avatar_url",
            "displayName": "display_name",
            "platformUserIds": "platform_user_ids",
            "userId": "user_id",
            "username": "username",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "avatarUrl": True,
            "displayName": True,
            "platformUserIds": True,
            "userId": True,
            "username": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service (4.25.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
import re
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class ItemTypeEnum(StrEnum):
    APP = "APP"
    BUNDLE = "BUNDLE"
    CODE = "CODE"
    COINS = "COINS"
    EXTENSION = "EXTENSION"
    INGAMEITEM = "INGAMEITEM"
    LOOTBOX = "LOOTBOX"
    MEDIA = "MEDIA"
    OPTIONBOX = "OPTIONBOX"
    SEASON = "SEASON"
    SUBSCRIPTION = "SUBSCRIPTION"


class PaymentOrderCreate(Model):
    """Payment order create (PaymentOrderCreate)

    Properties:
        description: (description) REQUIRED str

        ext_order_no: (extOrderNo) REQUIRED str

        price: (price) REQUIRED int

        title: (title) REQUIRED str

        currency_code: (currencyCode) OPTIONAL str

        currency_namespace: (currencyNamespace) OPTIONAL str

        custom_parameters: (customParameters) OPTIONAL Dict[str, Any]

        ext_user_id: (extUserId) OPTIONAL str

        item_type: (itemType) OPTIONAL Union[str, ItemTypeEnum]

        language: (language) OPTIONAL str

        metadata: (metadata) OPTIONAL Dict[str, str]

        notify_url: (notifyUrl) OPTIONAL str

        omit_notification: (omitNotification) OPTIONAL bool

        platform: (platform) OPTIONAL str

        recurring_payment_order_no: (recurringPaymentOrderNo) OPTIONAL str

        region: (region) OPTIONAL str

        return_url: (returnUrl) OPTIONAL str

        sandbox: (sandbox) OPTIONAL bool

        sku: (sku) OPTIONAL str

        subscription_id: (subscriptionId) OPTIONAL str
    """

    # region fields

    description: str  # REQUIRED
    ext_order_no: str  # REQUIRED
    price: int  # REQUIRED
    title: str  # REQUIRED
    currency_code: str  # OPTIONAL
    currency_namespace: str  # OPTIONAL
    custom_parameters: Dict[str, Any]  # OPTIONAL
    ext_user_id: str  # OPTIONAL
    item_type: Union[str, ItemTypeEnum]  # OPTIONAL
    language: str  # OPTIONAL
    metadata: Dict[str, str]  # OPTIONAL
    notify_url: str  # OPTIONAL
    omit_notification: bool  # OPTIONAL
    platform: str  # OPTIONAL
    recurring_payment_order_no: str  # OPTIONAL
    region: str  # OPTIONAL
    return_url: str  # OPTIONAL
    sandbox: bool  # OPTIONAL
    sku: str  # OPTIONAL
    subscription_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_description(self, value: str) -> PaymentOrderCreate:
        self.description = value
        return self

    def with_ext_order_no(self, value: str) -> PaymentOrderCreate:
        self.ext_order_no = value
        return self

    def with_price(self, value: int) -> PaymentOrderCreate:
        self.price = value
        return self

    def with_title(self, value: str) -> PaymentOrderCreate:
        self.title = value
        return self

    def with_currency_code(self, value: str) -> PaymentOrderCreate:
        self.currency_code = value
        return self

    def with_currency_namespace(self, value: str) -> PaymentOrderCreate:
        self.currency_namespace = value
        return self

    def with_custom_parameters(self, value: Dict[str, Any]) -> PaymentOrderCreate:
        self.custom_parameters = value
        return self

    def with_ext_user_id(self, value: str) -> PaymentOrderCreate:
        self.ext_user_id = value
        return self

    def with_item_type(self, value: Union[str, ItemTypeEnum]) -> PaymentOrderCreate:
        self.item_type = value
        return self

    def with_language(self, value: str) -> PaymentOrderCreate:
        self.language = value
        return self

    def with_metadata(self, value: Dict[str, str]) -> PaymentOrderCreate:
        self.metadata = value
        return self

    def with_notify_url(self, value: str) -> PaymentOrderCreate:
        self.notify_url = value
        return self

    def with_omit_notification(self, value: bool) -> PaymentOrderCreate:
        self.omit_notification = value
        return self

    def with_platform(self, value: str) -> PaymentOrderCreate:
        self.platform = value
        return self

    def with_recurring_payment_order_no(self, value: str) -> PaymentOrderCreate:
        self.recurring_payment_order_no = value
        return self

    def with_region(self, value: str) -> PaymentOrderCreate:
        self.region = value
        return self

    def with_return_url(self, value: str) -> PaymentOrderCreate:
        self.return_url = value
        return self

    def with_sandbox(self, value: bool) -> PaymentOrderCreate:
        self.sandbox = value
        return self

    def with_sku(self, value: str) -> PaymentOrderCreate:
        self.sku = value
        return self

    def with_subscription_id(self, value: str) -> PaymentOrderCreate:
        self.subscription_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "ext_order_no"):
            result["extOrderNo"] = str(self.ext_order_no)
        elif include_empty:
            result["extOrderNo"] = ""
        if hasattr(self, "price"):
            result["price"] = int(self.price)
        elif include_empty:
            result["price"] = 0
        if hasattr(self, "title"):
            result["title"] = str(self.title)
        elif include_empty:
            result["title"] = ""
        if hasattr(self, "currency_code"):
            result["currencyCode"] = str(self.currency_code)
        elif include_empty:
            result["currencyCode"] = ""
        if hasattr(self, "currency_namespace"):
            result["currencyNamespace"] = str(self.currency_namespace)
        elif include_empty:
            result["currencyNamespace"] = ""
        if hasattr(self, "custom_parameters"):
            result["customParameters"] = {
                str(k0): v0 for k0, v0 in self.custom_parameters.items()
            }
        elif include_empty:
            result["customParameters"] = {}
        if hasattr(self, "ext_user_id"):
            result["extUserId"] = str(self.ext_user_id)
        elif include_empty:
            result["extUserId"] = ""
        if hasattr(self, "item_type"):
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = Union[str, ItemTypeEnum]()
        if hasattr(self, "language"):
            result["language"] = str(self.language)
        elif include_empty:
            result["language"] = ""
        if hasattr(self, "metadata"):
            result["metadata"] = {str(k0): str(v0) for k0, v0 in self.metadata.items()}
        elif include_empty:
            result["metadata"] = {}
        if hasattr(self, "notify_url"):
            result["notifyUrl"] = str(self.notify_url)
        elif include_empty:
            result["notifyUrl"] = ""
        if hasattr(self, "omit_notification"):
            result["omitNotification"] = bool(self.omit_notification)
        elif include_empty:
            result["omitNotification"] = False
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = ""
        if hasattr(self, "recurring_payment_order_no"):
            result["recurringPaymentOrderNo"] = str(self.recurring_payment_order_no)
        elif include_empty:
            result["recurringPaymentOrderNo"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "return_url"):
            result["returnUrl"] = str(self.return_url)
        elif include_empty:
            result["returnUrl"] = ""
        if hasattr(self, "sandbox"):
            result["sandbox"] = bool(self.sandbox)
        elif include_empty:
            result["sandbox"] = False
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "subscription_id"):
            result["subscriptionId"] = str(self.subscription_id)
        elif include_empty:
            result["subscriptionId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        description: str,
        ext_order_no: str,
        price: int,
        title: str,
        currency_code: Optional[str] = None,
        currency_namespace: Optional[str] = None,
        custom_parameters: Optional[Dict[str, Any]] = None,
        ext_user_id: Optional[str] = None,
        item_type: Optional[Union[str, ItemTypeEnum]] = None,
        language: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        notify_url: Optional[str] = None,
        omit_notification: Optional[bool] = None,
        platform: Optional[str] = None,
        recurring_payment_order_no: Optional[str] = None,
        region: Optional[str] = None,
        return_url: Optional[str] = None,
        sandbox: Optional[bool] = None,
        sku: Optional[str] = None,
        subscription_id: Optional[str] = None,
    ) -> PaymentOrderCreate:
        instance = cls()
        instance.description = description
        instance.ext_order_no = ext_order_no
        instance.price = price
        instance.title = title
        if currency_code is not None:
            instance.currency_code = currency_code
        if currency_namespace is not None:
            instance.currency_namespace = currency_namespace
        if custom_parameters is not None:
            instance.custom_parameters = custom_parameters
        if ext_user_id is not None:
            instance.ext_user_id = ext_user_id
        if item_type is not None:
            instance.item_type = item_type
        if language is not None:
            instance.language = language
        if metadata is not None:
            instance.metadata = metadata
        if notify_url is not None:
            instance.notify_url = notify_url
        if omit_notification is not None:
            instance.omit_notification = omit_notification
        if platform is not None:
            instance.platform = platform
        if recurring_payment_order_no is not None:
            instance.recurring_payment_order_no = recurring_payment_order_no
        if region is not None:
            instance.region = region
        if return_url is not None:
            instance.return_url = return_url
        if sandbox is not None:
            instance.sandbox = sandbox
        if sku is not None:
            instance.sku = sku
        if subscription_id is not None:
            instance.subscription_id = subscription_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PaymentOrderCreate:
        instance = cls()
        if not dict_:
            return instance
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "extOrderNo" in dict_ and dict_["extOrderNo"] is not None:
            instance.ext_order_no = str(dict_["extOrderNo"])
        elif include_empty:
            instance.ext_order_no = ""
        if "price" in dict_ and dict_["price"] is not None:
            instance.price = int(dict_["price"])
        elif include_empty:
            instance.price = 0
        if "title" in dict_ and dict_["title"] is not None:
            instance.title = str(dict_["title"])
        elif include_empty:
            instance.title = ""
        if "currencyCode" in dict_ and dict_["currencyCode"] is not None:
            instance.currency_code = str(dict_["currencyCode"])
        elif include_empty:
            instance.currency_code = ""
        if "currencyNamespace" in dict_ and dict_["currencyNamespace"] is not None:
            instance.currency_namespace = str(dict_["currencyNamespace"])
        elif include_empty:
            instance.currency_namespace = ""
        if "customParameters" in dict_ and dict_["customParameters"] is not None:
            instance.custom_parameters = {
                str(k0): v0 for k0, v0 in dict_["customParameters"].items()
            }
        elif include_empty:
            instance.custom_parameters = {}
        if "extUserId" in dict_ and dict_["extUserId"] is not None:
            instance.ext_user_id = str(dict_["extUserId"])
        elif include_empty:
            instance.ext_user_id = ""
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = Union[str, ItemTypeEnum]()
        if "language" in dict_ and dict_["language"] is not None:
            instance.language = str(dict_["language"])
        elif include_empty:
            instance.language = ""
        if "metadata" in dict_ and dict_["metadata"] is not None:
            instance.metadata = {
                str(k0): str(v0) for k0, v0 in dict_["metadata"].items()
            }
        elif include_empty:
            instance.metadata = {}
        if "notifyUrl" in dict_ and dict_["notifyUrl"] is not None:
            instance.notify_url = str(dict_["notifyUrl"])
        elif include_empty:
            instance.notify_url = ""
        if "omitNotification" in dict_ and dict_["omitNotification"] is not None:
            instance.omit_notification = bool(dict_["omitNotification"])
        elif include_empty:
            instance.omit_notification = False
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = ""
        if (
            "recurringPaymentOrderNo" in dict_
            and dict_["recurringPaymentOrderNo"] is not None
        ):
            instance.recurring_payment_order_no = str(dict_["recurringPaymentOrderNo"])
        elif include_empty:
            instance.recurring_payment_order_no = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "returnUrl" in dict_ and dict_["returnUrl"] is not None:
            instance.return_url = str(dict_["returnUrl"])
        elif include_empty:
            instance.return_url = ""
        if "sandbox" in dict_ and dict_["sandbox"] is not None:
            instance.sandbox = bool(dict_["sandbox"])
        elif include_empty:
            instance.sandbox = False
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "subscriptionId" in dict_ and dict_["subscriptionId"] is not None:
            instance.subscription_id = str(dict_["subscriptionId"])
        elif include_empty:
            instance.subscription_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PaymentOrderCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PaymentOrderCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        PaymentOrderCreate, List[PaymentOrderCreate], Dict[Any, PaymentOrderCreate]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "description": "description",
            "extOrderNo": "ext_order_no",
            "price": "price",
            "title": "title",
            "currencyCode": "currency_code",
            "currencyNamespace": "currency_namespace",
            "customParameters": "custom_parameters",
            "extUserId": "ext_user_id",
            "itemType": "item_type",
            "language": "language",
            "metadata": "metadata",
            "notifyUrl": "notify_url",
            "omitNotification": "omit_notification",
            "platform": "platform",
            "recurringPaymentOrderNo": "recurring_payment_order_no",
            "region": "region",
            "returnUrl": "return_url",
            "sandbox": "sandbox",
            "sku": "sku",
            "subscriptionId": "subscription_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "description": True,
            "extOrderNo": True,
            "price": True,
            "title": True,
            "currencyCode": False,
            "currencyNamespace": False,
            "customParameters": False,
            "extUserId": False,
            "itemType": False,
            "language": False,
            "metadata": False,
            "notifyUrl": False,
            "omitNotification": False,
            "platform": False,
            "recurringPaymentOrderNo": False,
            "region": False,
            "returnUrl": False,
            "sandbox": False,
            "sku": False,
            "subscriptionId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemType": [
                "APP",
                "BUNDLE",
                "CODE",
                "COINS",
                "EXTENSION",
                "INGAMEITEM",
                "LOOTBOX",
                "MEDIA",
                "OPTIONBOX",
                "SEASON",
                "SUBSCRIPTION",
            ],
        }

    @staticmethod
    def get_pattern_map() -> Dict[str, re.Pattern]:
        return {
            "language": re.compile(
                r"^[A-Za-z]{2,4}([_-][A-Za-z]{4})?([_-]([A-Za-z]{2}|[0-9]{3}))?$"
            ),
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Browser Service ()

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.sessionbrowser.models import ModelsActiveCustomGameResponse
from ..api.sessionbrowser.models import ModelsActiveMatchmakingGameResponse
from ..api.sessionbrowser.models import ModelsAddPlayerRequest
from ..api.sessionbrowser.models import ModelsAddPlayerResponse
from ..api.sessionbrowser.models import ModelsAdminSessionResponse
from ..api.sessionbrowser.models import ModelsCountActiveSessionResponse
from ..api.sessionbrowser.models import ModelsCreateSessionRequest
from ..api.sessionbrowser.models import ModelsCustomGameResponse
from ..api.sessionbrowser.models import ModelsGameSession
from ..api.sessionbrowser.models import ModelsGameSessionSetting
from ..api.sessionbrowser.models import ModelsGetSessionHistoryDetailedResponseItem
from ..api.sessionbrowser.models import ModelsGetSessionHistorySearchResponseItemV2
from ..api.sessionbrowser.models import ModelsGetSessionHistorySearchResponseV2
from ..api.sessionbrowser.models import ModelsJoinGameSessionRequest
from ..api.sessionbrowser.models import ModelsMatchAttributes
from ..api.sessionbrowser.models import ModelsMatchMaking
from ..api.sessionbrowser.models import ModelsMatchingAlly
from ..api.sessionbrowser.models import ModelsMatchingParty
from ..api.sessionbrowser.models import ModelsPagination
from ..api.sessionbrowser.models import ModelsPagingCursor
from ..api.sessionbrowser.models import ModelsPartyMember
from ..api.sessionbrowser.models import ModelsRecentPlayerHistory
from ..api.sessionbrowser.models import ModelsRecentPlayerQueryResponse
from ..api.sessionbrowser.models import ModelsServer
from ..api.sessionbrowser.models import ModelsSessionByUserIDsResponse
from ..api.sessionbrowser.models import ModelsSessionPlayerHistory
from ..api.sessionbrowser.models import ModelsSessionPlayerJoining
from ..api.sessionbrowser.models import ModelsSessionQueryResponse
from ..api.sessionbrowser.models import ModelsSessionResponse
from ..api.sessionbrowser.models import ModelsStatusHistory
from ..api.sessionbrowser.models import ModelsUpdateSessionRequest
from ..api.sessionbrowser.models import ModelsUpdateSettingsRequest
from ..api.sessionbrowser.models import ResponseError
from ..api.sessionbrowser.models import RestapiErrorResponseV2
from ..api.sessionbrowser.models import RestapiErrorV1


def create_models_active_custom_game_response_example() -> ModelsActiveCustomGameResponse:
    instance = ModelsActiveCustomGameResponse()
    instance.pagination = create_models_paging_cursor_example()
    instance.sessions = [create_models_custom_game_response_example()]
    return instance


def create_models_active_matchmaking_game_response_example() -> ModelsActiveMatchmakingGameResponse:
    instance = ModelsActiveMatchmakingGameResponse()
    instance.pagination = create_models_paging_cursor_example()
    instance.sessions = [create_models_game_session_example()]
    return instance


def create_models_add_player_request_example() -> ModelsAddPlayerRequest:
    instance = ModelsAddPlayerRequest()
    instance.as_spectator = randomize("bool")
    instance.user_id = randomize("uid")
    return instance


def create_models_add_player_response_example() -> ModelsAddPlayerResponse:
    instance = ModelsAddPlayerResponse()
    instance.status = randomize("bool")
    return instance


def create_models_admin_session_response_example() -> ModelsAdminSessionResponse:
    instance = ModelsAdminSessionResponse()
    instance.all_players = [randomize()]
    instance.created_at = randomize("date")
    instance.game_session_setting = create_models_game_session_setting_example()
    instance.game_version = randomize("version")
    instance.joinable = randomize("bool")
    instance.match = create_models_match_making_example()
    instance.namespace = randomize("slug")
    instance.players = [randomize()]
    instance.server = create_models_server_example()
    instance.session_id = randomize("uid")
    instance.session_type = randomize()
    instance.spectators = [randomize()]
    instance.user_id = randomize("uid")
    instance.username = randomize("slug")
    return instance


def create_models_count_active_session_response_example() -> ModelsCountActiveSessionResponse:
    instance = ModelsCountActiveSessionResponse()
    instance.custom_game = randomize("int", min_val=1, max_val=1000)
    instance.matchmaking_game = randomize("int", min_val=1, max_val=1000)
    instance.total = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_create_session_request_example() -> ModelsCreateSessionRequest:
    instance = ModelsCreateSessionRequest()
    instance.game_session_setting = create_models_game_session_setting_example()
    instance.game_version = randomize("version")
    instance.namespace = randomize("slug")
    instance.session_type = randomize()
    instance.username = randomize("slug")
    return instance


def create_models_custom_game_response_example() -> ModelsCustomGameResponse:
    instance = ModelsCustomGameResponse()
    instance.all_players = [randomize()]
    instance.created_at = randomize("date")
    instance.game_session_setting = create_models_game_session_setting_example()
    instance.joinable = randomize("bool")
    instance.namespace = randomize("slug")
    instance.players = [randomize()]
    instance.server = create_models_server_example()
    instance.session_id = randomize("uid")
    instance.session_type = randomize()
    instance.spectators = [randomize()]
    return instance


def create_models_game_session_example() -> ModelsGameSession:
    instance = ModelsGameSession()
    instance.all_players = [randomize()]
    instance.created_at = randomize("date")
    instance.game_session_setting = create_models_game_session_setting_example()
    instance.game_version = randomize("version")
    instance.joinable = randomize("bool")
    instance.match = create_models_match_making_example()
    instance.namespace = randomize("slug")
    instance.players = [randomize()]
    instance.server = create_models_server_example()
    instance.session_id = randomize("uid")
    instance.session_type = randomize()
    instance.spectators = [randomize()]
    instance.user_id = randomize("uid")
    instance.username = randomize("slug")
    return instance


def create_models_game_session_setting_example() -> ModelsGameSessionSetting:
    instance = ModelsGameSessionSetting()
    instance.allow_join_in_progress = randomize("bool")
    instance.current_internal_player = randomize("int", min_val=1, max_val=1000)
    instance.current_player = randomize("int", min_val=1, max_val=1000)
    instance.map_name = randomize()
    instance.max_internal_player = randomize("int", min_val=1, max_val=1000)
    instance.max_player = randomize("int", min_val=1, max_val=1000)
    instance.mode = randomize()
    instance.num_bot = randomize("int", min_val=1, max_val=1000)
    instance.password = randomize("password")
    instance.settings = {randomize(): randomize()}
    return instance


def create_models_get_session_history_detailed_response_item_example() -> ModelsGetSessionHistoryDetailedResponseItem:
    instance = ModelsGetSessionHistoryDetailedResponseItem()
    instance.channel = randomize()
    instance.client_version = randomize()
    instance.created_at = randomize("date")
    instance.event_description = randomize()
    instance.event_name = randomize()
    instance.game_mode = randomize()
    instance.id_ = randomize()
    instance.joinable = randomize("bool")
    instance.match_id = randomize()
    instance.matching_allies = [create_models_matching_ally_example()]
    instance.namespace = randomize("slug")
    instance.region = randomize()
    instance.server_name = randomize()
    instance.status = randomize()
    instance.party_id = randomize("uid")
    return instance


def create_models_get_session_history_search_response_item_v2_example() -> ModelsGetSessionHistorySearchResponseItemV2:
    instance = ModelsGetSessionHistorySearchResponseItemV2()
    instance.created_at = randomize("date")
    instance.game_mode = randomize()
    instance.id_ = randomize()
    instance.joinable = randomize("bool")
    instance.joining = [create_models_session_player_joining_example()]
    instance.match_id = randomize()
    instance.namespace = randomize("slug")
    instance.players = [create_models_session_player_history_example()]
    instance.removed_reason = randomize()
    instance.session_type = randomize()
    instance.status = randomize()
    instance.sub_game_mode = [randomize()]
    instance.user_id = randomize("uid")
    return instance


def create_models_get_session_history_search_response_v2_example() -> ModelsGetSessionHistorySearchResponseV2:
    instance = ModelsGetSessionHistorySearchResponseV2()
    instance.data = [
        create_models_get_session_history_search_response_item_v2_example()
    ]
    instance.pagination = create_models_pagination_example()
    return instance


def create_models_join_game_session_request_example() -> ModelsJoinGameSessionRequest:
    instance = ModelsJoinGameSessionRequest()
    instance.password = randomize("password")
    return instance


def create_models_match_attributes_example() -> ModelsMatchAttributes:
    instance = ModelsMatchAttributes()
    instance.first_ticket_created_at = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_match_making_example() -> ModelsMatchMaking:
    instance = ModelsMatchMaking()
    instance.channel = randomize()
    instance.client_version = randomize()
    instance.created_at = randomize("date")
    instance.deployment = randomize()
    instance.event = randomize()
    instance.game_mode = randomize()
    instance.match_id = randomize()
    instance.matching_allies = [create_models_matching_ally_example()]
    instance.namespace = randomize("slug")
    instance.party_attributes = {randomize(): randomize()}
    instance.queued_at = randomize("int", min_val=1, max_val=1000)
    instance.region = randomize()
    instance.server_name = randomize()
    instance.status = randomize()
    instance.sub_game_mode = [randomize()]
    instance.joinable = randomize("bool")
    instance.party_id = randomize("uid")
    return instance


def create_models_matching_ally_example() -> ModelsMatchingAlly:
    instance = ModelsMatchingAlly()
    instance.matching_parties = [create_models_matching_party_example()]
    return instance


def create_models_matching_party_example() -> ModelsMatchingParty:
    instance = ModelsMatchingParty()
    instance.match_attributes = create_models_match_attributes_example()
    instance.party_attributes = {randomize(): randomize()}
    instance.party_id = randomize("uid")
    instance.party_members = [create_models_party_member_example()]
    return instance


def create_models_pagination_example() -> ModelsPagination:
    instance = ModelsPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_paging_cursor_example() -> ModelsPagingCursor:
    instance = ModelsPagingCursor()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_party_member_example() -> ModelsPartyMember:
    instance = ModelsPartyMember()
    instance.extra_attributes = {randomize(): randomize()}
    instance.user_id = randomize("uid")
    return instance


def create_models_recent_player_history_example() -> ModelsRecentPlayerHistory:
    instance = ModelsRecentPlayerHistory()
    instance.created_at = randomize("date")
    instance.namespace = randomize("slug")
    instance.other_display_name = randomize()
    instance.other_id = randomize()
    instance.updated_at = randomize("date")
    instance.user_id = randomize("uid")
    return instance


def create_models_recent_player_query_response_example() -> ModelsRecentPlayerQueryResponse:
    instance = ModelsRecentPlayerQueryResponse()
    instance.data = [create_models_recent_player_history_example()]
    return instance


def create_models_server_example() -> ModelsServer:
    instance = ModelsServer()
    instance.allocation_id = randomize()
    instance.alternate_ips = [randomize()]
    instance.cpu_limit = randomize("int", min_val=1, max_val=1000)
    instance.cpu_request = randomize()
    instance.deployment = randomize()
    instance.game_version = randomize("version")
    instance.image_version = randomize()
    instance.ip = randomize()
    instance.is_override_game_version = randomize("bool")
    instance.last_update = randomize("date")
    instance.mem_limit = randomize("int", min_val=1, max_val=1000)
    instance.mem_request = randomize()
    instance.namespace = randomize("slug")
    instance.params = randomize()
    instance.pod_name = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.ports = {}
    instance.provider = randomize()
    instance.region = randomize()
    instance.session_id = randomize("uid")
    instance.status = randomize()
    instance.status_history = [create_models_status_history_example()]
    return instance


def create_models_session_by_user_i_ds_response_example() -> ModelsSessionByUserIDsResponse:
    instance = ModelsSessionByUserIDsResponse()
    instance.data = [create_models_game_session_example()]
    return instance


def create_models_session_player_history_example() -> ModelsSessionPlayerHistory:
    instance = ModelsSessionPlayerHistory()
    instance.action = randomize()
    instance.date = randomize("date")
    instance.user_id = randomize("uid")
    return instance


def create_models_session_player_joining_example() -> ModelsSessionPlayerJoining:
    instance = ModelsSessionPlayerJoining()
    instance.date = randomize("date")
    instance.state = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_session_query_response_example() -> ModelsSessionQueryResponse:
    instance = ModelsSessionQueryResponse()
    instance.pagination = create_models_paging_cursor_example()
    instance.sessions = [create_models_game_session_example()]
    return instance


def create_models_session_response_example() -> ModelsSessionResponse:
    instance = ModelsSessionResponse()
    instance.all_players = [randomize()]
    instance.created_at = randomize("date")
    instance.game_session_setting = create_models_game_session_setting_example()
    instance.game_version = randomize("version")
    instance.joinable = randomize("bool")
    instance.match = create_models_match_making_example()
    instance.namespace = randomize("slug")
    instance.players = [randomize()]
    instance.server = create_models_server_example()
    instance.session_id = randomize("uid")
    instance.session_type = randomize()
    instance.spectators = [randomize()]
    instance.user_id = randomize("uid")
    instance.username = randomize("slug")
    return instance


def create_models_status_history_example() -> ModelsStatusHistory:
    instance = ModelsStatusHistory()
    instance.status = randomize()
    instance.time_stamp = randomize("date")
    return instance


def create_models_update_session_request_example() -> ModelsUpdateSessionRequest:
    instance = ModelsUpdateSessionRequest()
    instance.game_max_player = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_settings_request_example() -> ModelsUpdateSettingsRequest:
    instance = ModelsUpdateSettingsRequest()
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance


def create_restapi_error_response_v2_example() -> RestapiErrorResponseV2:
    instance = RestapiErrorResponseV2()
    instance.attributes = {randomize(): randomize()}
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.message = randomize()
    instance.name = randomize()
    return instance


def create_restapi_error_v1_example() -> RestapiErrorV1:
    instance = RestapiErrorV1()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Qo S Manager Service ()

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.qosm.models import ModelsHeartbeatRequest
from ..api.qosm.models import ModelsListServerResponse
from ..api.qosm.models import ModelsServer
from ..api.qosm.models import ModelsSetAliasRequest
from ..api.qosm.models import ModelsUpdateServerRequest
from ..api.qosm.models import ResponseError


def create_models_heartbeat_request_example() -> ModelsHeartbeatRequest:
    instance = ModelsHeartbeatRequest()
    instance.ip = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.region = randomize()
    return instance


def create_models_list_server_response_example() -> ModelsListServerResponse:
    instance = ModelsListServerResponse()
    instance.servers = [create_models_server_example()]
    return instance


def create_models_server_example() -> ModelsServer:
    instance = ModelsServer()
    instance.alias = randomize()
    instance.ip = randomize()
    instance.last_update = randomize("date")
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.region = randomize()
    instance.status = randomize()
    return instance


def create_models_set_alias_request_example() -> ModelsSetAliasRequest:
    instance = ModelsSetAliasRequest()
    instance.alias = randomize()
    return instance


def create_models_update_server_request_example() -> ModelsUpdateServerRequest:
    instance = ModelsUpdateServerRequest()
    instance.status = randomize()
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.code = randomize("int", min_val=1, max_val=1000)
    instance.message = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsClaimSessionRequest
from ..models import ModelsCreateSessionRequest
from ..models import ModelsSessionResponse
from ..models import ResponseError

from ..operations.session import ClaimServer
from ..operations.session import CreateSession
from ..operations.session import GetSession


@same_doc_as(ClaimServer)
def claim_server(
    body: ModelsClaimSessionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Claim a DS for a game session (ClaimServer)

    Required permission: NAMESPACE:{namespace}:DSM:SESSION [UPDATE]

    Required scope: social

    This endpoint is intended to be called by game session manager (matchmaker, lobby, etc.) to claim a dedicated server. The dedicated server cannot be claimed unless the status is READY

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:SESSION [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/sessions/claim

        method: POST

        tags: ["Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsClaimSessionRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (DS claimed for session)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (session not found)

        409: Conflict - ResponseError (DS is already claimed)

        425: Too Early - ResponseError (DS is not ready to be claimed)

        500: Internal Server Error - ResponseError (Internal Server Error)

        503: Service Unavailable - ResponseError (DS is unreachable)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ClaimServer.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ClaimServer)
async def claim_server_async(
    body: ModelsClaimSessionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Claim a DS for a game session (ClaimServer)

    Required permission: NAMESPACE:{namespace}:DSM:SESSION [UPDATE]

    Required scope: social

    This endpoint is intended to be called by game session manager (matchmaker, lobby, etc.) to claim a dedicated server. The dedicated server cannot be claimed unless the status is READY

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:SESSION [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/sessions/claim

        method: POST

        tags: ["Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsClaimSessionRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (DS claimed for session)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (session not found)

        409: Conflict - ResponseError (DS is already claimed)

        425: Too Early - ResponseError (DS is not ready to be claimed)

        500: Internal Server Error - ResponseError (Internal Server Error)

        503: Service Unavailable - ResponseError (DS is unreachable)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ClaimServer.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateSession)
def create_session(
    body: ModelsCreateSessionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Register a new game session (CreateSession)

    Required permission: NAMESPACE:{namespace}:DSM:SESSION [CREATE]

    Required scope: social

    This endpoint is intended to be called by game session manager (matchmaker, lobby, etc.) to get a dedicated server for a game session.

    If a dedicated server is available, it will respond with a dedicated server details ready to be used.

    Otherwise it will trigger new dedicated server creation and respond with a server status CREATING. The game session manager then expected to wait and query the server readiness with GET /namespaces/{namespace}/sessions/{sessionID} endpoint until the serverstatus is READY

    Specify pod_name with name of local DS in the request to create a session using the registered local DS

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:SESSION [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/sessions

        method: POST

        tags: ["Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateSessionRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsSessionResponse (session created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (local server not found)

        409: Conflict - ResponseError (session already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)

        503: Service Unavailable - ResponseError (server count is at max)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateSession.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateSession)
async def create_session_async(
    body: ModelsCreateSessionRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Register a new game session (CreateSession)

    Required permission: NAMESPACE:{namespace}:DSM:SESSION [CREATE]

    Required scope: social

    This endpoint is intended to be called by game session manager (matchmaker, lobby, etc.) to get a dedicated server for a game session.

    If a dedicated server is available, it will respond with a dedicated server details ready to be used.

    Otherwise it will trigger new dedicated server creation and respond with a server status CREATING. The game session manager then expected to wait and query the server readiness with GET /namespaces/{namespace}/sessions/{sessionID} endpoint until the serverstatus is READY

    Specify pod_name with name of local DS in the request to create a session using the registered local DS

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:SESSION [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/sessions

        method: POST

        tags: ["Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateSessionRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsSessionResponse (session created)

        400: Bad Request - ResponseError (malformed request)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (local server not found)

        409: Conflict - ResponseError (session already exists)

        500: Internal Server Error - ResponseError (Internal Server Error)

        503: Service Unavailable - ResponseError (server count is at max)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateSession.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetSession)
def get_session(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query specified session (GetSession)

    Required permission: NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required scope: social

    This endpoint is intended to be called by game session manager (matchmaker, lobby, etc.) to query the status of dedicated server that is created for the session.

    The server is ready to use when the status is READY. At which point, the game session manager can claim the server using the GET /namespaces/{namespace}/sessions/{sessionID}/claim endpoint

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/sessions/{sessionID}

        method: GET

        tags: ["Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in path

    Responses:
        200: OK - ModelsSessionResponse (session queried)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (session not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSession.create(
        session_id=session_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetSession)
async def get_session_async(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query specified session (GetSession)

    Required permission: NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required scope: social

    This endpoint is intended to be called by game session manager (matchmaker, lobby, etc.) to query the status of dedicated server that is created for the session.

    The server is ready to use when the status is READY. At which point, the game session manager can claim the server using the GET /namespaces/{namespace}/sessions/{sessionID}/claim endpoint

    Required Permission(s):
        - NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/namespaces/{namespace}/sessions/{sessionID}

        method: GET

        tags: ["Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in path

    Responses:
        200: OK - ModelsSessionResponse (session queried)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (session not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSession.create(
        session_id=session_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Match Service V2 (2.2.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.api_player_data import ApiPlayerData


class ApiTicket(Model):
    """Api ticket (api.Ticket)

    Properties:
        created_at: (createdAt) OPTIONAL str

        latencies: (latencies) OPTIONAL Dict[str, int]

        match_pool: (matchPool) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        party_session_id: (partySessionID) OPTIONAL str

        players: (players) OPTIONAL List[ApiPlayerData]

        ticket_attributes: (ticketAttributes) OPTIONAL Dict[str, Any]

        ticket_id: (ticketID) OPTIONAL str
    """

    # region fields

    created_at: str  # OPTIONAL
    latencies: Dict[str, int]  # OPTIONAL
    match_pool: str  # OPTIONAL
    namespace: str  # OPTIONAL
    party_session_id: str  # OPTIONAL
    players: List[ApiPlayerData]  # OPTIONAL
    ticket_attributes: Dict[str, Any]  # OPTIONAL
    ticket_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> ApiTicket:
        self.created_at = value
        return self

    def with_latencies(self, value: Dict[str, int]) -> ApiTicket:
        self.latencies = value
        return self

    def with_match_pool(self, value: str) -> ApiTicket:
        self.match_pool = value
        return self

    def with_namespace(self, value: str) -> ApiTicket:
        self.namespace = value
        return self

    def with_party_session_id(self, value: str) -> ApiTicket:
        self.party_session_id = value
        return self

    def with_players(self, value: List[ApiPlayerData]) -> ApiTicket:
        self.players = value
        return self

    def with_ticket_attributes(self, value: Dict[str, Any]) -> ApiTicket:
        self.ticket_attributes = value
        return self

    def with_ticket_id(self, value: str) -> ApiTicket:
        self.ticket_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "latencies"):
            result["latencies"] = {
                str(k0): int(v0) for k0, v0 in self.latencies.items()
            }
        elif include_empty:
            result["latencies"] = {}
        if hasattr(self, "match_pool"):
            result["matchPool"] = str(self.match_pool)
        elif include_empty:
            result["matchPool"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "party_session_id"):
            result["partySessionID"] = str(self.party_session_id)
        elif include_empty:
            result["partySessionID"] = ""
        if hasattr(self, "players"):
            result["players"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.players
            ]
        elif include_empty:
            result["players"] = []
        if hasattr(self, "ticket_attributes"):
            result["ticketAttributes"] = {
                str(k0): v0 for k0, v0 in self.ticket_attributes.items()
            }
        elif include_empty:
            result["ticketAttributes"] = {}
        if hasattr(self, "ticket_id"):
            result["ticketID"] = str(self.ticket_id)
        elif include_empty:
            result["ticketID"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: Optional[str] = None,
        latencies: Optional[Dict[str, int]] = None,
        match_pool: Optional[str] = None,
        namespace: Optional[str] = None,
        party_session_id: Optional[str] = None,
        players: Optional[List[ApiPlayerData]] = None,
        ticket_attributes: Optional[Dict[str, Any]] = None,
        ticket_id: Optional[str] = None,
    ) -> ApiTicket:
        instance = cls()
        if created_at is not None:
            instance.created_at = created_at
        if latencies is not None:
            instance.latencies = latencies
        if match_pool is not None:
            instance.match_pool = match_pool
        if namespace is not None:
            instance.namespace = namespace
        if party_session_id is not None:
            instance.party_session_id = party_session_id
        if players is not None:
            instance.players = players
        if ticket_attributes is not None:
            instance.ticket_attributes = ticket_attributes
        if ticket_id is not None:
            instance.ticket_id = ticket_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> ApiTicket:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "latencies" in dict_ and dict_["latencies"] is not None:
            instance.latencies = {
                str(k0): int(v0) for k0, v0 in dict_["latencies"].items()
            }
        elif include_empty:
            instance.latencies = {}
        if "matchPool" in dict_ and dict_["matchPool"] is not None:
            instance.match_pool = str(dict_["matchPool"])
        elif include_empty:
            instance.match_pool = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "partySessionID" in dict_ and dict_["partySessionID"] is not None:
            instance.party_session_id = str(dict_["partySessionID"])
        elif include_empty:
            instance.party_session_id = ""
        if "players" in dict_ and dict_["players"] is not None:
            instance.players = [
                ApiPlayerData.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["players"]
            ]
        elif include_empty:
            instance.players = []
        if "ticketAttributes" in dict_ and dict_["ticketAttributes"] is not None:
            instance.ticket_attributes = {
                str(k0): v0 for k0, v0 in dict_["ticketAttributes"].items()
            }
        elif include_empty:
            instance.ticket_attributes = {}
        if "ticketID" in dict_ and dict_["ticketID"] is not None:
            instance.ticket_id = str(dict_["ticketID"])
        elif include_empty:
            instance.ticket_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiTicket]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiTicket]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApiTicket, List[ApiTicket], Dict[Any, ApiTicket]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "latencies": "latencies",
            "matchPool": "match_pool",
            "namespace": "namespace",
            "partySessionID": "party_session_id",
            "players": "players",
            "ticketAttributes": "ticket_attributes",
            "ticketID": "ticket_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": False,
            "latencies": False,
            "matchPool": False,
            "namespace": False,
            "partySessionID": False,
            "players": False,
            "ticketAttributes": False,
            "ticketID": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Gdpr Service (1.19.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsDeletionStatus(Model):
    """Models deletion status (models.DeletionStatus)

    Properties:
        deletion_status: (DeletionStatus) REQUIRED bool

        display_name: (DisplayName) REQUIRED str

        status: (Status) REQUIRED str

        user_id: (UserID) REQUIRED str

        execution_date: (ExecutionDate) OPTIONAL str
    """

    # region fields

    deletion_status: bool  # REQUIRED
    display_name: str  # REQUIRED
    status: str  # REQUIRED
    user_id: str  # REQUIRED
    execution_date: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_deletion_status(self, value: bool) -> ModelsDeletionStatus:
        self.deletion_status = value
        return self

    def with_display_name(self, value: str) -> ModelsDeletionStatus:
        self.display_name = value
        return self

    def with_status(self, value: str) -> ModelsDeletionStatus:
        self.status = value
        return self

    def with_user_id(self, value: str) -> ModelsDeletionStatus:
        self.user_id = value
        return self

    def with_execution_date(self, value: str) -> ModelsDeletionStatus:
        self.execution_date = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "deletion_status"):
            result["DeletionStatus"] = bool(self.deletion_status)
        elif include_empty:
            result["DeletionStatus"] = False
        if hasattr(self, "display_name"):
            result["DisplayName"] = str(self.display_name)
        elif include_empty:
            result["DisplayName"] = ""
        if hasattr(self, "status"):
            result["Status"] = str(self.status)
        elif include_empty:
            result["Status"] = ""
        if hasattr(self, "user_id"):
            result["UserID"] = str(self.user_id)
        elif include_empty:
            result["UserID"] = ""
        if hasattr(self, "execution_date"):
            result["ExecutionDate"] = str(self.execution_date)
        elif include_empty:
            result["ExecutionDate"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        deletion_status: bool,
        display_name: str,
        status: str,
        user_id: str,
        execution_date: Optional[str] = None,
    ) -> ModelsDeletionStatus:
        instance = cls()
        instance.deletion_status = deletion_status
        instance.display_name = display_name
        instance.status = status
        instance.user_id = user_id
        if execution_date is not None:
            instance.execution_date = execution_date
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsDeletionStatus:
        instance = cls()
        if not dict_:
            return instance
        if "DeletionStatus" in dict_ and dict_["DeletionStatus"] is not None:
            instance.deletion_status = bool(dict_["DeletionStatus"])
        elif include_empty:
            instance.deletion_status = False
        if "DisplayName" in dict_ and dict_["DisplayName"] is not None:
            instance.display_name = str(dict_["DisplayName"])
        elif include_empty:
            instance.display_name = ""
        if "Status" in dict_ and dict_["Status"] is not None:
            instance.status = str(dict_["Status"])
        elif include_empty:
            instance.status = ""
        if "UserID" in dict_ and dict_["UserID"] is not None:
            instance.user_id = str(dict_["UserID"])
        elif include_empty:
            instance.user_id = ""
        if "ExecutionDate" in dict_ and dict_["ExecutionDate"] is not None:
            instance.execution_date = str(dict_["ExecutionDate"])
        elif include_empty:
            instance.execution_date = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsDeletionStatus]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsDeletionStatus]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsDeletionStatus,
        List[ModelsDeletionStatus],
        Dict[Any, ModelsDeletionStatus],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "DeletionStatus": "deletion_status",
            "DisplayName": "display_name",
            "Status": "status",
            "UserID": "user_id",
            "ExecutionDate": "execution_date",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "DeletionStatus": True,
            "DisplayName": True,
            "Status": True,
            "UserID": True,
            "ExecutionDate": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Dsm Controller Service (6.2.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_pod_count_config_override import ModelsPodCountConfigOverride


class ModelsCreateDeploymentOverrideRequest(Model):
    """Models create deployment override request (models.CreateDeploymentOverrideRequest)

    Properties:
        buffer_count: (buffer_count) REQUIRED int

        buffer_percent: (buffer_percent) REQUIRED int

        configuration: (configuration) REQUIRED str

        enable_region_overrides: (enable_region_overrides) REQUIRED bool

        game_version: (game_version) REQUIRED str

        max_count: (max_count) REQUIRED int

        min_count: (min_count) REQUIRED int

        region_overrides: (region_overrides) REQUIRED Dict[str, ModelsPodCountConfigOverride]

        regions: (regions) REQUIRED List[str]

        unlimited: (unlimited) REQUIRED bool

        use_buffer_percent: (use_buffer_percent) REQUIRED bool

        session_timeout: (session_timeout) OPTIONAL int
    """

    # region fields

    buffer_count: int  # REQUIRED
    buffer_percent: int  # REQUIRED
    configuration: str  # REQUIRED
    enable_region_overrides: bool  # REQUIRED
    game_version: str  # REQUIRED
    max_count: int  # REQUIRED
    min_count: int  # REQUIRED
    region_overrides: Dict[str, ModelsPodCountConfigOverride]  # REQUIRED
    regions: List[str]  # REQUIRED
    unlimited: bool  # REQUIRED
    use_buffer_percent: bool  # REQUIRED
    session_timeout: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_buffer_count(self, value: int) -> ModelsCreateDeploymentOverrideRequest:
        self.buffer_count = value
        return self

    def with_buffer_percent(self, value: int) -> ModelsCreateDeploymentOverrideRequest:
        self.buffer_percent = value
        return self

    def with_configuration(self, value: str) -> ModelsCreateDeploymentOverrideRequest:
        self.configuration = value
        return self

    def with_enable_region_overrides(
        self, value: bool
    ) -> ModelsCreateDeploymentOverrideRequest:
        self.enable_region_overrides = value
        return self

    def with_game_version(self, value: str) -> ModelsCreateDeploymentOverrideRequest:
        self.game_version = value
        return self

    def with_max_count(self, value: int) -> ModelsCreateDeploymentOverrideRequest:
        self.max_count = value
        return self

    def with_min_count(self, value: int) -> ModelsCreateDeploymentOverrideRequest:
        self.min_count = value
        return self

    def with_region_overrides(
        self, value: Dict[str, ModelsPodCountConfigOverride]
    ) -> ModelsCreateDeploymentOverrideRequest:
        self.region_overrides = value
        return self

    def with_regions(self, value: List[str]) -> ModelsCreateDeploymentOverrideRequest:
        self.regions = value
        return self

    def with_unlimited(self, value: bool) -> ModelsCreateDeploymentOverrideRequest:
        self.unlimited = value
        return self

    def with_use_buffer_percent(
        self, value: bool
    ) -> ModelsCreateDeploymentOverrideRequest:
        self.use_buffer_percent = value
        return self

    def with_session_timeout(self, value: int) -> ModelsCreateDeploymentOverrideRequest:
        self.session_timeout = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "buffer_count"):
            result["buffer_count"] = int(self.buffer_count)
        elif include_empty:
            result["buffer_count"] = 0
        if hasattr(self, "buffer_percent"):
            result["buffer_percent"] = int(self.buffer_percent)
        elif include_empty:
            result["buffer_percent"] = 0
        if hasattr(self, "configuration"):
            result["configuration"] = str(self.configuration)
        elif include_empty:
            result["configuration"] = ""
        if hasattr(self, "enable_region_overrides"):
            result["enable_region_overrides"] = bool(self.enable_region_overrides)
        elif include_empty:
            result["enable_region_overrides"] = False
        if hasattr(self, "game_version"):
            result["game_version"] = str(self.game_version)
        elif include_empty:
            result["game_version"] = ""
        if hasattr(self, "max_count"):
            result["max_count"] = int(self.max_count)
        elif include_empty:
            result["max_count"] = 0
        if hasattr(self, "min_count"):
            result["min_count"] = int(self.min_count)
        elif include_empty:
            result["min_count"] = 0
        if hasattr(self, "region_overrides"):
            result["region_overrides"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.region_overrides.items()
            }
        elif include_empty:
            result["region_overrides"] = {}
        if hasattr(self, "regions"):
            result["regions"] = [str(i0) for i0 in self.regions]
        elif include_empty:
            result["regions"] = []
        if hasattr(self, "unlimited"):
            result["unlimited"] = bool(self.unlimited)
        elif include_empty:
            result["unlimited"] = False
        if hasattr(self, "use_buffer_percent"):
            result["use_buffer_percent"] = bool(self.use_buffer_percent)
        elif include_empty:
            result["use_buffer_percent"] = False
        if hasattr(self, "session_timeout"):
            result["session_timeout"] = int(self.session_timeout)
        elif include_empty:
            result["session_timeout"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        buffer_count: int,
        buffer_percent: int,
        configuration: str,
        enable_region_overrides: bool,
        game_version: str,
        max_count: int,
        min_count: int,
        region_overrides: Dict[str, ModelsPodCountConfigOverride],
        regions: List[str],
        unlimited: bool,
        use_buffer_percent: bool,
        session_timeout: Optional[int] = None,
    ) -> ModelsCreateDeploymentOverrideRequest:
        instance = cls()
        instance.buffer_count = buffer_count
        instance.buffer_percent = buffer_percent
        instance.configuration = configuration
        instance.enable_region_overrides = enable_region_overrides
        instance.game_version = game_version
        instance.max_count = max_count
        instance.min_count = min_count
        instance.region_overrides = region_overrides
        instance.regions = regions
        instance.unlimited = unlimited
        instance.use_buffer_percent = use_buffer_percent
        if session_timeout is not None:
            instance.session_timeout = session_timeout
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsCreateDeploymentOverrideRequest:
        instance = cls()
        if not dict_:
            return instance
        if "buffer_count" in dict_ and dict_["buffer_count"] is not None:
            instance.buffer_count = int(dict_["buffer_count"])
        elif include_empty:
            instance.buffer_count = 0
        if "buffer_percent" in dict_ and dict_["buffer_percent"] is not None:
            instance.buffer_percent = int(dict_["buffer_percent"])
        elif include_empty:
            instance.buffer_percent = 0
        if "configuration" in dict_ and dict_["configuration"] is not None:
            instance.configuration = str(dict_["configuration"])
        elif include_empty:
            instance.configuration = ""
        if (
            "enable_region_overrides" in dict_
            and dict_["enable_region_overrides"] is not None
        ):
            instance.enable_region_overrides = bool(dict_["enable_region_overrides"])
        elif include_empty:
            instance.enable_region_overrides = False
        if "game_version" in dict_ and dict_["game_version"] is not None:
            instance.game_version = str(dict_["game_version"])
        elif include_empty:
            instance.game_version = ""
        if "max_count" in dict_ and dict_["max_count"] is not None:
            instance.max_count = int(dict_["max_count"])
        elif include_empty:
            instance.max_count = 0
        if "min_count" in dict_ and dict_["min_count"] is not None:
            instance.min_count = int(dict_["min_count"])
        elif include_empty:
            instance.min_count = 0
        if "region_overrides" in dict_ and dict_["region_overrides"] is not None:
            instance.region_overrides = {
                str(k0): ModelsPodCountConfigOverride.create_from_dict(
                    v0, include_empty=include_empty
                )
                for k0, v0 in dict_["region_overrides"].items()
            }
        elif include_empty:
            instance.region_overrides = {}
        if "regions" in dict_ and dict_["regions"] is not None:
            instance.regions = [str(i0) for i0 in dict_["regions"]]
        elif include_empty:
            instance.regions = []
        if "unlimited" in dict_ and dict_["unlimited"] is not None:
            instance.unlimited = bool(dict_["unlimited"])
        elif include_empty:
            instance.unlimited = False
        if "use_buffer_percent" in dict_ and dict_["use_buffer_percent"] is not None:
            instance.use_buffer_percent = bool(dict_["use_buffer_percent"])
        elif include_empty:
            instance.use_buffer_percent = False
        if "session_timeout" in dict_ and dict_["session_timeout"] is not None:
            instance.session_timeout = int(dict_["session_timeout"])
        elif include_empty:
            instance.session_timeout = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsCreateDeploymentOverrideRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsCreateDeploymentOverrideRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsCreateDeploymentOverrideRequest,
        List[ModelsCreateDeploymentOverrideRequest],
        Dict[Any, ModelsCreateDeploymentOverrideRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "buffer_count": "buffer_count",
            "buffer_percent": "buffer_percent",
            "configuration": "configuration",
            "enable_region_overrides": "enable_region_overrides",
            "game_version": "game_version",
            "max_count": "max_count",
            "min_count": "min_count",
            "region_overrides": "region_overrides",
            "regions": "regions",
            "unlimited": "unlimited",
            "use_buffer_percent": "use_buffer_percent",
            "session_timeout": "session_timeout",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "buffer_count": True,
            "buffer_percent": True,
            "configuration": True,
            "enable_region_overrides": True,
            "game_version": True,
            "max_count": True,
            "min_count": True,
            "region_overrides": True,
            "regions": True,
            "unlimited": True,
            "use_buffer_percent": True,
            "session_timeout": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "1.19.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_cancel_user_accou_de1ba5 import AdminCancelUserAccountDeletionRequest
from .admin_get_list_deletion_227d7d import AdminGetListDeletionDataRequest
from .admin_get_user_account__84ef5a import AdminGetUserAccountDeletionRequest
from .admin_submit_user_accou_b7780f import AdminSubmitUserAccountDeletionRequest
from .public_cancel_user_acco_16b962 import PublicCancelUserAccountDeletionRequest
from .public_get_user_account_09e4f2 import PublicGetUserAccountDeletionStatus
from .public_submit_user_acco_a6db4f import PublicSubmitUserAccountDeletionRequest



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApiBackFillAcceptRequest
from ..models import ApiBackFillCreateRequest
from ..models import ApiBackFillRejectRequest
from ..models import ApiBackfillCreateResponse
from ..models import ApiBackfillGetResponse
from ..models import ApiBackfillProposalResponse
from ..models import ResponseError

from ..operations.backfill import AcceptBackfill
from ..operations.backfill import CreateBackfill
from ..operations.backfill import DeleteBackfill
from ..operations.backfill import GetBackfill
from ..operations.backfill import GetBackfillProposal
from ..operations.backfill import RejectBackfill


@same_doc_as(AcceptBackfill)
def accept_backfill(
    backfill_id: str,
    body: ApiBackFillAcceptRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Accept a backfill proposal (AcceptBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope: social

    Accept backfill proposal

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/accept

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillAcceptRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AcceptBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AcceptBackfill)
async def accept_backfill_async(
    backfill_id: str,
    body: ApiBackFillAcceptRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Accept a backfill proposal (AcceptBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope: social

    Accept backfill proposal

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/accept

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillAcceptRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AcceptBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateBackfill)
def create_backfill(
    body: ApiBackFillCreateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a backfill ticket (CreateBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [CREATE]

    Required Scope: social

    Create backfill ticket

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill

        method: POST

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillCreateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApiBackfillCreateResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateBackfill.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateBackfill)
async def create_backfill_async(
    body: ApiBackFillCreateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a backfill ticket (CreateBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [CREATE]

    Required Scope: social

    Create backfill ticket

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [CREATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill

        method: POST

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillCreateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApiBackfillCreateResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateBackfill.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteBackfill)
def delete_backfill(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a backfill ticket (DeleteBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [DELETE]

    Required Scope: social

    Delete backfill ticket

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: DELETE

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteBackfill)
async def delete_backfill_async(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a backfill ticket (DeleteBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [DELETE]

    Required Scope: social

    Delete backfill ticket

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: DELETE

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetBackfill)
def get_backfill(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a backfill ticket (GetBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [GET]

    Required Scope: social

    Get backfill ticket by ID

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL []

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApiBackfillGetResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetBackfill)
async def get_backfill_async(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a backfill ticket (GetBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [GET]

    Required Scope: social

    Get backfill ticket by ID

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL []

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApiBackfillGetResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetBackfillProposal)
def get_backfill_proposal(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get backfill proposal (GetBackfillProposal)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [GET]

    Required Scope: social

    Get backfill proposal

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL []

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/proposal

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in query

    Responses:
        200: OK - ApiBackfillProposalResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetBackfillProposal.create(
        session_id=session_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetBackfillProposal)
async def get_backfill_proposal_async(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get backfill proposal (GetBackfillProposal)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [GET]

    Required Scope: social

    Get backfill proposal

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL []

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/proposal

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in query

    Responses:
        200: OK - ApiBackfillProposalResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetBackfillProposal.create(
        session_id=session_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RejectBackfill)
def reject_backfill(
    backfill_id: str,
    body: ApiBackFillRejectRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Reject a backfill proposal (RejectBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope: social

    Reject backfill proposal

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/reject

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillRejectRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RejectBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RejectBackfill)
async def reject_backfill_async(
    backfill_id: str,
    body: ApiBackFillRejectRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Reject a backfill proposal (RejectBackfill)

    Required Permission: NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope: social

    Reject backfill proposal

    Required Permission(s):
        - NAMESPACE:{namespace}:MATCHMAKING:BACKFILL [UPDATE]

    Required Scope(s):
        - social

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/reject

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillRejectRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RejectBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )
